﻿SET client_encoding = 'UTF8';

--- DROP & CREATE TABLE --------------------------------------------------------
DROP TABLE public.fiz_liczo;
DROP TABLE public.yur_liczo;
DROP TABLE public.dokument_licnost;
DROP TABLE public.organ_vlasti_ili_iyul;
DROP TABLE msk.zona;
DROP TABLE public.zona_territorialnay;
DROP TABLE public.zona_osobye_usloviya;
DROP TABLE public.dokument_osnovanie;

CREATE TABLE yur_liczo
(
  guid uuid NOT NULL DEFAULT uuid_generate_v4(),
  nazvanie text NOT NULL,
  id_opf character(12),
  inn character(10),
  elektronnaya_pochta character varying(50),
  kod_kpp character(9),
  kod_ogrn character(20),
  telefon character varying(20),
  strana character varying(255),
  kontakt_informaciya character varying(512),
  data_gos_registracii date,
  organ_registracii character varying(255),
  CONSTRAINT yur_liczo_pkey PRIMARY KEY (guid )
)
WITH (
  OIDS=FALSE
);
ALTER TABLE yur_liczo
  OWNER TO openlandadmin;
GRANT ALL ON TABLE yur_liczo TO openlandadmin;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE yur_liczo TO openlandusers;

CREATE TABLE fiz_liczo
(
  guid uuid NOT NULL DEFAULT uuid_generate_v4(),
  familiya character varying(45) NOT NULL,
  imya character varying(45) NOT NULL,
  otchestvo character varying(45),
  data_rozhdeniya date,
  mesto_rozhdeniya character varying(255),
  id_dokument_lichnost integer,
  elektronnaya_pochta character varying(50),
  telefon character varying(20),
  semeiynoe_pologenie bit(1),
  graghdanstvo character varying(255),
  nesovershennoletniy boolean,
  kontaktnaya_informaciya character varying(512),
  pol character(1),
  snils character(20),
  CONSTRAINT fiz_liczo_pkey PRIMARY KEY (guid )
)
WITH (
  OIDS=FALSE
);
ALTER TABLE fiz_liczo
  OWNER TO openlandadmin;
GRANT ALL ON TABLE fiz_liczo TO openlandadmin;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE fiz_liczo TO openlandusers;


--- CREATE TABLE ---------------------------------------------------------------
CREATE TABLE class.section_mp
(
  id serial NOT NULL,
  id_parent integer NOT NULL DEFAULT 0,
  name character varying(100) NOT NULL,
  xml_name character varying(40) NOT NULL,
  sort_order SMALLINT,
  number_version_mp SMALLINT NOT NULL,
  CONSTRAINT section_mp_pkey PRIMARY KEY (id ),
  CONSTRAINT section_mp_xml_name_key UNIQUE (xml_name, number_version_mp)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE class.section_mp
  OWNER TO openlandadmin;
GRANT ALL ON TABLE class.section_mp TO openlandadmin;
GRANT SELECT ON TABLE class.section_mp TO openlandusers;
GRANT SELECT ON TABLE class.section_mp TO openlandusers;


CREATE TABLE mp
(
  guid uuid NOT NULL DEFAULT uuid_generate_v4(),
  codetype character(3) NOT NULL,
  version character(2) NOT NULL,
  date_work date,
  purpose text,
  reason text,
  guid_contractor uuid,
  date_accept date,
  conclusion text,
  guid_client uuid,
  CONSTRAINT mp_pkey PRIMARY KEY (guid )
)
WITH (
  OIDS=FALSE
);
ALTER TABLE mp
  OWNER TO openlandadmin;
GRANT ALL ON TABLE mp TO openlandadmin;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE mp TO openlandusers;

CREATE TABLE contractor
(
  guid uuid NOT NULL DEFAULT uuid_generate_v4(),
  surname character varying(45) NOT NULL,
  first character varying(90) NOT NULL,
  patronymic character varying(45) NOT NULL,
  n_certificate character varying(50) NOT NULL,
  telephone character varying(50) NOT NULL,
  address character varying(250) NOT NULL,
  email character varying(100) NOT NULL,
  organization character varying(255) NOT NULL,
  CONSTRAINT contractor_pkey PRIMARY KEY (guid )
)
WITH (
  OIDS=FALSE
);
ALTER TABLE contractor
  OWNER TO openlandadmin;
GRANT ALL ON TABLE contractor TO openlandadmin;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE contractor TO openlandusers;

CREATE TABLE client
(
  guid uuid NOT NULL DEFAULT uuid_generate_v4(),
  guid_fiz_liczo uuid,
  guid_yur_liczo uuid,
  guid_governance uuid,
  guid_foreign_organization uuid,
  guid_agent uuid,
  appointment character varying(256),
  CONSTRAINT client_pkey PRIMARY KEY (guid )
)
WITH (
  OIDS=FALSE
);
ALTER TABLE client
  OWNER TO openlandadmin;
GRANT ALL ON TABLE client TO openlandadmin;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE client TO openlandusers;

CREATE TABLE foreign_organization
(
  guid uuid NOT NULL DEFAULT uuid_generate_v4(),
  name text NOT NULL,
  country character varying(256),
  CONSTRAINT foreign_organization_pkey PRIMARY KEY (guid )
)
WITH (
  OIDS=FALSE
);
ALTER TABLE foreign_organization
  OWNER TO openlandadmin;
GRANT ALL ON TABLE foreign_organization TO openlandadmin;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE foreign_organization TO openlandusers;

CREATE TABLE governance
(
  guid uuid NOT NULL DEFAULT uuid_generate_v4(),
  name text NOT NULL,
  CONSTRAINT governance_pkey PRIMARY KEY (guid )
)
WITH (
  OIDS=FALSE
);
ALTER TABLE governance
  OWNER TO openlandadmin;
GRANT ALL ON TABLE governance TO openlandadmin;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE governance TO openlandusers;

CREATE TABLE mp_section
(
  guid uuid NOT NULL DEFAULT uuid_generate_v4(),
  id_section_mp integer NOT NULL,
  guid_mp uuid NOT NULL,
  CONSTRAINT mp_section_pkey PRIMARY KEY (guid )
)
WITH (
  OIDS=FALSE
);
ALTER TABLE mp_section
  OWNER TO openlandadmin;
GRANT ALL ON TABLE mp_section TO openlandadmin;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE mp_section TO openlandusers;

CREATE TABLE applied_file
(
  guid uuid NOT NULL DEFAULT uuid_generate_v4(),
  kind character(2) NOT NULL DEFAULT '01'::bpchar,
  name character varying(255) NOT NULL,
  path character varying(255) NOT NULL,
  CONSTRAINT file_applied_pkey PRIMARY KEY (guid )
)
WITH (
  OIDS=FALSE
);
ALTER TABLE applied_file
  OWNER TO openlandadmin;
GRANT ALL ON TABLE applied_file TO openlandadmin;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE applied_file TO openlandusers;

CREATE TABLE document
(
  guid uuid NOT NULL DEFAULT uuid_generate_v4(),
  code_document character(12) NOT NULL,
  name character varying(255),
  series character varying(45),
  number_doc character varying(45) NOT NULL,
  date date,
  issue_organ character varying(255),
  number_reg character varying(45),
  date_reg date,
  duration_started date,
  duration_stopped date,
  register character varying(255),
  desc_doc character varying(255),
  issue_organ_code character varying(20),
  CONSTRAINT document_pkey PRIMARY KEY (guid )
)
WITH (
  OIDS=FALSE
);
ALTER TABLE document
  OWNER TO openlandadmin;
GRANT ALL ON TABLE document TO openlandadmin;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE document TO openlandusers;

CREATE TABLE mp_section_data
(
  guid uuid NOT NULL DEFAULT uuid_generate_v4(),
  guid_mp UUID NOT NULL,
  id_section_mp integer NOT NULL,
  id_parcel integer,
  guid_document uuid,
  guid_geodesic_base uuid,
  guid_means_survey uuid,
  guid_oks uuid,
  guid_subparcel uuid,
  guid_applied_file uuid,
  CONSTRAINT mp_section_document_pkey PRIMARY KEY (guid )
)
WITH (
  OIDS=FALSE
);
ALTER TABLE mp_section_data
  OWNER TO openlandadmin;
GRANT ALL ON TABLE mp_section_data TO openlandadmin;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE mp_section_data TO openlandusers;

CREATE TABLE document_applied_file
(
  guid uuid NOT NULL DEFAULT uuid_generate_v4(),
  guid_document uuid,
  guid_file uuid,
  CONSTRAINT document_applied_file_pkey PRIMARY KEY (guid )
)
WITH (
  OIDS=FALSE
);
ALTER TABLE document_applied_file
  OWNER TO openlandadmin;
GRANT ALL ON TABLE document_applied_file TO openlandadmin;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE document_applied_file TO openlandusers;

CREATE TABLE oks
(
  guid uuid NOT NULL DEFAULT uuid_generate_v4(),
  cadastral_number character varying(40) NOT NULL,
  cadastral_number_other_number text NOT NULL,
  name_oks text,
  CONSTRAINT oks_pkey PRIMARY KEY (guid )
)
WITH (
  OIDS=FALSE
);
ALTER TABLE oks
  OWNER TO openlandadmin;
GRANT ALL ON TABLE oks TO openlandadmin;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE oks TO openlandusers;

CREATE TABLE subparcel
(
  guid uuid NOT NULL DEFAULT uuid_generate_v4(),
  cadastral_number character varying(40) NOT NULL,
  number_record text NOT NULL,
  CONSTRAINT subparcel_pkey PRIMARY KEY (guid )
)
WITH (
  OIDS=FALSE
);
ALTER TABLE subparcel
  OWNER TO openlandadmin;
GRANT ALL ON TABLE subparcel TO openlandadmin;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE subparcel TO openlandusers;

CREATE TABLE duplicate_text_values
(
  guid uuid NOT NULL DEFAULT uuid_generate_v4(),
  text_type smallint NOT NULL,
  text_value text NOT NULL,
  CONSTRAINT duplicate_text_values_pkey PRIMARY KEY (guid )
)
WITH (
  OIDS=FALSE
);
ALTER TABLE duplicate_text_values
  OWNER TO openlandadmin;
GRANT ALL ON TABLE duplicate_text_values TO openlandadmin;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE duplicate_text_values TO openlandusers;

CREATE TABLE class.type_applied_file
(
  id character(2) NOT NULL,
  name character varying(40),
  CONSTRAINT type_applied_file_pkey PRIMARY KEY (id )
)
WITH (
  OIDS=FALSE
);
ALTER TABLE class.type_applied_file
  OWNER TO openlandadmin;
GRANT ALL ON TABLE class.type_applied_file TO openlandadmin;
GRANT SELECT ON TABLE class.type_applied_file TO openlandusers;

CREATE TABLE kn_document
(
  guid uuid NOT NULL DEFAULT uuid_generate_v4(),
  id_kn integer NOT NULL,
  guid_document uuid NOT NULL,
  CONSTRAINT kn_document_pkey PRIMARY KEY (guid )
)
WITH (
  OIDS=FALSE
);
ALTER TABLE kn_document
  OWNER TO openlandadmin;
GRANT ALL ON TABLE kn_document TO openlandadmin;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE kn_document TO openlandusers;


--- ALTER TABLE - ADD COLUMN ---------------------------------------------------
ALTER TABLE public.kn
  ADD COLUMN definition VARCHAR(50);

ALTER TABLE public.kn
  ADD COLUMN other VARCHAR(50);

ALTER TABLE public.kn
  ADD COLUMN number VARCHAR(50);

ALTER TABLE public.adres
  ADD COLUMN guid_document UUID;

ALTER TABLE msk.uchastok
  ADD COLUMN guid_doc_category UUID;

ALTER TABLE msk.uchastok
  ADD COLUMN guid_doc_utilization UUID;

ALTER TABLE msk.uchastok
  ADD COLUMN min_area NUMERIC(20,2);

ALTER TABLE msk.uchastok
  ADD COLUMN type_unit_min_area CHAR(4);

ALTER TABLE msk.uchastok
  ADD COLUMN max_area NUMERIC(20,2);

ALTER TABLE msk.uchastok
  ADD COLUMN type_unit_max_area CHAR(4);

ALTER TABLE msk.tochka
  ADD COLUMN id_child INTEGER;


--- ALTER COLUMN TYPE ----------------------------------------------------------
ALTER TABLE public.fiz_liczo
  ALTER COLUMN semeiynoe_pologenie TYPE CHAR(1);
                 

--- ALTER COLUMN DROP NOT NULL -------------------------------------------------
ALTER TABLE public.kn
  ALTER COLUMN kn DROP NOT NULL;


--- ALTER RENAME COLUMN --------------------------------------------------------
ALTER TABLE public.sredstva_izmereniya
  RENAME COLUMN id TO guid;
ALTER TABLE sredstva_izmereniya ALTER COLUMN guid TYPE uuid USING
  CAST(regexp_replace(guid, '([A-Z0-9]{4})([A-Z0-9]{12})', E'\\1-\\2') AS uuid);

ALTER TABLE public.geo_osnova
  RENAME COLUMN id TO guid;
ALTER TABLE public.geo_osnova ALTER COLUMN guid TYPE uuid USING
  CAST(regexp_replace(guid, '([A-Z0-9]{4})([A-Z0-9]{12})', E'\\1-\\2') AS uuid);


--- ALTER GRANT ----------------------------------------------------------------
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE msk.rayon TO openlandusers;

           
--- ALTER ADD CONSTRAINT -------------------------------------------------------
ALTER TABLE public.mp
  ADD CONSTRAINT mp_fk FOREIGN KEY (guid_contractor)
    REFERENCES public.contractor(guid)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

ALTER TABLE public.mp
  ADD CONSTRAINT mp_fk1 FOREIGN KEY (guid_client)
    REFERENCES public.client(guid)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

ALTER TABLE public.client
  ADD CONSTRAINT client_fk FOREIGN KEY (guid_fiz_liczo)
    REFERENCES public.fiz_liczo(guid)
    ON DELETE CASCADE
    ON UPDATE CASCADE
    NOT DEFERRABLE;

ALTER TABLE public.client
  ADD CONSTRAINT client_fk1 FOREIGN KEY (guid_yur_liczo)
    REFERENCES public.yur_liczo(guid)
    ON DELETE CASCADE
    ON UPDATE CASCADE
    NOT DEFERRABLE;

ALTER TABLE public.client
  ADD CONSTRAINT client_fk2 FOREIGN KEY (guid_governance)
    REFERENCES public.governance(guid)
    ON DELETE CASCADE
    ON UPDATE CASCADE
    NOT DEFERRABLE;

ALTER TABLE public.client
  ADD CONSTRAINT client_fk3 FOREIGN KEY (guid_foreign_organization)
    REFERENCES public.foreign_organization(guid)
    ON DELETE CASCADE
    ON UPDATE CASCADE
    NOT DEFERRABLE;


ALTER TABLE public.client
  ADD CONSTRAINT client_fk4 FOREIGN KEY (guid_agent)
    REFERENCES public.fiz_liczo(guid)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

ALTER TABLE public.document
  ADD CONSTRAINT document_fk FOREIGN KEY (code_document)
    REFERENCES class.dokument(id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

ALTER TABLE public.document_applied_file
  ADD CONSTRAINT document_applied_file_fk FOREIGN KEY (guid_document)
    REFERENCES public.document(guid)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

ALTER TABLE public.document_applied_file
  ADD CONSTRAINT document_applied_file_fk1 FOREIGN KEY (guid_file)
    REFERENCES public.applied_file(guid)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

ALTER TABLE public.mp_section_data
  ADD CONSTRAINT mp_section_data_fk FOREIGN KEY (guid_mp)
    REFERENCES public.mp(guid)
    ON DELETE CASCADE
    ON UPDATE CASCADE
    NOT DEFERRABLE;

ALTER TABLE public.mp_section_data
  ADD CONSTRAINT mp_section_data_fk1 FOREIGN KEY (guid_document)
    REFERENCES public.document(guid)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

ALTER TABLE public.mp_section_data
  ADD CONSTRAINT mp_section_data_fk2 FOREIGN KEY (guid_geodesic_base)
    REFERENCES public.geo_osnova(guid)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

ALTER TABLE public.mp_section_data
  ADD CONSTRAINT mp_section_data_fk3 FOREIGN KEY (guid_means_survey)
    REFERENCES public.sredstva_izmereniya(guid)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

ALTER TABLE public.mp_section_data
  ADD CONSTRAINT mp_section_data_fk4 FOREIGN KEY (guid_oks)
    REFERENCES public.oks(guid)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

ALTER TABLE public.mp_section_data
  ADD CONSTRAINT mp_section_data_fk5 FOREIGN KEY (guid_subparcel)
    REFERENCES public.subparcel(guid)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

ALTER TABLE public.mp_section_data
  ADD CONSTRAINT mp_section_data_fk6 FOREIGN KEY (guid_applied_file)
    REFERENCES public.applied_file(guid)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

ALTER TABLE public.mp_section_data
  ADD CONSTRAINT mp_section_data_fk7 FOREIGN KEY (id_parcel)
    REFERENCES msk.uchastok(id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

ALTER TABLE public.mp_section_data
  ADD CONSTRAINT mp_section_data_fk8 FOREIGN KEY (id_section_mp)
    REFERENCES class.section_mp(id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

CREATE UNIQUE INDEX mp_section_data_idx ON public.mp_section_data
  USING btree (guid_mp, id_parcel);

CREATE UNIQUE INDEX mp_section_data_idx1 ON public.mp_section_data
  USING btree (guid_mp, id_section_mp, guid_document);

ALTER TABLE public.applied_file
  ADD CONSTRAINT applied_file_fk FOREIGN KEY (kind)
    REFERENCES class.type_applied_file(id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

ALTER TABLE public.kn_document
  ADD CONSTRAINT kn_document_fk FOREIGN KEY (id_kn)
    REFERENCES public.kn(id)
    ON DELETE CASCADE
    ON UPDATE CASCADE
    NOT DEFERRABLE;

ALTER TABLE public.kn_document
  ADD CONSTRAINT kn_document_fk1 FOREIGN KEY (guid_document)
    REFERENCES public.document(guid)
    ON DELETE CASCADE
    ON UPDATE CASCADE
    NOT DEFERRABLE;

ALTER TABLE public.adres
  ADD CONSTRAINT adres_fk11 FOREIGN KEY (guid_document)
    REFERENCES public.document(guid)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

ALTER TABLE msk.uchastok
  ADD CONSTRAINT uchastok_fk10 FOREIGN KEY (guid_doc_category)
    REFERENCES public.document(guid)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

ALTER TABLE msk.uchastok
  ADD CONSTRAINT uchastok_fk11 FOREIGN KEY (guid_doc_utilization)
    REFERENCES public.document(guid)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

ALTER TABLE msk.uchastok
  ADD CONSTRAINT uchastok_fk12 FOREIGN KEY (type_unit_min_area)
    REFERENCES class.edinicy_izmereniya(id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

ALTER TABLE msk.uchastok
  ADD CONSTRAINT uchastok_fk13 FOREIGN KEY (type_unit_max_area)
    REFERENCES class.edinicy_izmereniya(id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
    NOT DEFERRABLE;

CREATE UNIQUE INDEX tochka_idx1 ON msk.tochka
  USING btree (id, id_child);


--- public.mnogokonturnyy & public.chast => public.parcel_parcel ---------------
ALTER TABLE class.vid_zemelnogo_uchastka
  ALTER COLUMN znachenie TYPE VARCHAR(30) COLLATE pg_catalog."default";
INSERT INTO class.vid_zemelnogo_uchastka VALUES ('06', 'Контур многоконтурного ЗУ');
INSERT INTO class.vid_zemelnogo_uchastka VALUES ('07', 'Часть земельного участка');

UPDATE msk.uchastok SET id_vid_uchastka = '06' WHERE id IN (SELECT id_kontur FROM public.mnogokonturnyy);
UPDATE msk.uchastok SET id_vid_uchastka = '07' WHERE id IN (SELECT id_chast FROM public.chast);

CREATE TABLE parcel_parcel
(
  guid uuid NOT NULL DEFAULT uuid_generate_v4(),
  id_parent integer NOT NULL,
  id_children integer NOT NULL,
  CONSTRAINT parcel_parcel_pkey PRIMARY KEY (guid )
)
WITH (
  OIDS=FALSE
);
ALTER TABLE parcel_parcel
  OWNER TO openlandadmin;
GRANT ALL ON TABLE parcel_parcel TO openlandadmin;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE parcel_parcel TO openlandusers;

ALTER TABLE public.mnogokonturnyy
  DROP CONSTRAINT mnogokonturnyy_fk RESTRICT;
ALTER TABLE public.mnogokonturnyy
  DROP CONSTRAINT mnogokonturnyy_fk1 RESTRICT;
ALTER TABLE public.chast
  DROP CONSTRAINT chast_fk1 RESTRICT;
ALTER TABLE public.chast
  DROP CONSTRAINT chast_fk RESTRICT;

INSERT INTO public.parcel_parcel (id_parent, id_children) SELECT id_uchastok, id_kontur FROM public.mnogokonturnyy;
INSERT INTO public.parcel_parcel (id_parent, id_children) SELECT id_uchastok, id_chast FROM public.chast;

DROP TABLE public.mnogokonturnyy;
DROP TABLE public.chast;

ALTER TABLE public.parcel_parcel
  ADD CONSTRAINT parcel_parcel_fk FOREIGN KEY (id_parent)
    REFERENCES msk.uchastok(id)
    ON DELETE CASCADE
    ON UPDATE CASCADE
    NOT DEFERRABLE;                        

ALTER TABLE public.parcel_parcel
  ADD CONSTRAINT parcel_parcel_fk1 FOREIGN KEY (id_children)
    REFERENCES msk.uchastok(id)
    ON DELETE CASCADE
    ON UPDATE CASCADE
    NOT DEFERRABLE;


--- REPLACE FUNCTION -----------------------------------------------------------
CREATE OR REPLACE FUNCTION public.ol_toch_ins (
)
RETURNS trigger AS
$body$
DECLARE
  idu INTEGER;
  ngeom geometry;
  idt INTEGER;
  pob SMALLINT;
  idparent INTEGER;
  maxnomer SMALLINT;

BEGIN
  idu = NEW.id_uchastok;
  ngeom = NEW.geom;
  pob = NEW.poryadok_obhoda;
-- проверка существования точки у данного ЗУ с такой же геометрией
-- если есть такая точка, тогда только меняется порядок обхода, но INSERT NEW не происходит
  SELECT id FROM msk.tochka WHERE (id_uchastok = idu) AND (public.st_equals(ngeom, geom)) INTO idt;

  IF (idt > 0) THEN
    IF (pob > 0) THEN
      UPDATE msk.tochka SET poryadok_obhoda = pob WHERE id = idt;
    END IF;
    RETURN NULL;
    
  ELSE
-- для новых точек корректируем нумерацию  
    IF NEW.prefiks_nomera = 'н' THEN
      SELECT id_parent FROM public.parcel_parcel WHERE id_children = idu INTO idparent;

      IF (idparent IS NOT NULL) THEN
        SELECT MAX(nomer) FROM msk.tochka 
          WHERE id_uchastok IN (SELECT id_children FROM public.parcel_parcel WHERE (id_parent = idparent))
            AND (prefiks_nomera = 'н') INTO maxnomer;
      ELSE
        SELECT MAX(nomer) FROM msk.tochka WHERE (id_uchastok = idu) AND (prefiks_nomera = 'н') INTO maxnomer;
      END IF;  
      
      IF (maxnomer IS NULL) THEN
        NEW.nomer = 1;
      ELSE
        NEW.nomer = maxnomer + 1;
      END IF;
    END IF;  

    RETURN NEW;

  END IF;  
  
END;
$body$
LANGUAGE 'plpgsql'
VOLATILE
CALLED ON NULL INPUT
SECURITY INVOKER
COST 100;


CREATE OR REPLACE FUNCTION public.ol_uch_upd (
)
RETURNS trigger AS
$body$
DECLARE
  iduch INTEGER;
  idkat CHAR(12);
  nomk SMALLINT;
  geomUch geometry;
BEGIN
  -- если меняется категория земель у многоконтурного, значит меняется категория и у всех контуров
  iduch = NEW.id;
  IF ((OLD.id_kategoriya IS NULL) OR (NEW.id_kategoriya <> OLD.id_kategoriya)) AND (NEW.id_vid_uchastka = '05') THEN
    idkat = NEW.id_kategoriya;
    UPDATE msk.uchastok SET id_kategoriya = idkat 
      WHERE id IN (SELECT id_children FROM public.parcel_parcel WHERE id_parent = iduch);   
  END IF;
  -- если меняется геометрия ЗУ, происходит проверка на наличие точек, прекращающих существование
  IF NOT st_equals(NEW.geom, OLD.geom) THEN
    geomUch = NEW.geom;
    UPDATE msk.tochka SET pre = 1, poryadok_obhoda = 0
      WHERE id_uchastok = iduch AND 
        (st_disjoint(geom, geomUch) OR st_within(geom, geomUch)); 
  END IF;  
  
  NEW.create_user = OLD.create_user;
  NEW.create_date = OLD.create_date;
  NEW.update_user = current_user;
  NEW.update_date = current_timestamp;
  RETURN NEW;
END;
$body$
LANGUAGE 'plpgsql'
VOLATILE
CALLED ON NULL INPUT
SECURITY INVOKER
COST 100;


--- DELETE DATA ----------------------------------------------------------------
DELETE FROM adres WHERE id NOT IN (SELECT id_adres FROM public.uchastok_adres);


--- INSERT DATA ----------------------------------------------------------------
INSERT INTO class.obekt_kadastrovyh_rabot VALUES (5, 'Образуемый земельный участок');

INSERT INTO class.type_applied_file VALUES ('01', 'Образ документа');
INSERT INTO class.type_applied_file VALUES ('02', 'Электронный документ');

INSERT INTO class.section_mp VALUES (18, 17, 'Документы основания, использованные для подготовки межевого плана', 'Documents', 1, 4);
INSERT INTO class.section_mp VALUES (19, 17, 'Сведения о геодезической основе', 'Geodesic_Bases', 2, 4);
INSERT INTO class.section_mp VALUES (20, 17, 'Сведения о средствах измерения', 'Means_Survey', 3, 4);
INSERT INTO class.section_mp VALUES (21, 17, 'Сведения о наличии зданий, сооружений', 'Realty', 4, 4);
INSERT INTO class.section_mp VALUES (22, 17, 'Сведения о частях исходных, изменяемых или уточняемых ЗУ', 'SubParcels', 5, 4);
INSERT INTO class.section_mp VALUES (8, 1, 'Уточнение границ нескольких смежных ЗУ', 'SpecifyParcelsApproximal', 4, 4);
INSERT INTO class.section_mp VALUES (14, 0, 'Электронный документ', 'eDocument', 1, 4);
INSERT INTO class.section_mp VALUES (15, 0, 'Титульный лист', 'Title', 2, 4);
INSERT INTO class.section_mp VALUES (1, 0, 'Пакет информации', 'Package', 3, 4);
INSERT INTO class.section_mp VALUES (16, 0, 'Система координат', 'Coord_System', 4, 4);
INSERT INTO class.section_mp VALUES (17, 0, 'Исходные данные', 'Input_Data', 5, 4);
INSERT INTO class.section_mp VALUES (23, 0, 'Сведения о выполненных измерениях и расчетах', 'Survey', 6, 4);
INSERT INTO class.section_mp VALUES (24, 0, 'Заключение кадастрового инженера', 'Conclusion', 7, 4);
INSERT INTO class.section_mp VALUES (2, 1, 'Образование участков', 'FormParcels', 1, 4);
INSERT INTO class.section_mp VALUES (6, 1, 'Уточнение границ', 'SpecifyParcel', 2, 4);
INSERT INTO class.section_mp VALUES (7, 1, 'Сведения об образуемых частях', 'NewSubParcel', 3, 4);
INSERT INTO class.section_mp VALUES (3, 2, 'Сведения об образуемых ЗУ и их частях', 'NewParcel', 1, 4);
INSERT INTO class.section_mp VALUES (4, 2, 'Сведения об изменяемых ЗУ и их частях', 'ChangeParcel', 2, 4);
INSERT INTO class.section_mp VALUES (9, 8, 'Сведения об уточняемом участке, не являющимся ЕЗ, и его частях', 'ExistParcel-Approximal', 1, 4);
INSERT INTO class.section_mp VALUES (10, 8, 'Сведения об уточняемом ЕЗ и его частях', 'ExistEZ-Approximal', 2, 4);
INSERT INTO class.section_mp VALUES (26, 6, 'Сведения об уточняемом участке, не являющемся ЕЗ, и его частях', 'ExistParcel', 1, 4);
INSERT INTO class.section_mp VALUES (27, 6, 'Сведения об уточняемом ЕЗ и его частях', 'ExistEZ', 2, 4);
INSERT INTO class.section_mp VALUES (5, 2, 'Уточнение границ смежных участков', 'SpecifyRelatedParcel-Form', 3, 4);
INSERT INTO class.section_mp VALUES (28, 6, 'Уточнение границ смежных участков', 'SpecifyRelatedParcel-Specify', 3, 4);
INSERT INTO class.section_mp VALUES (29, 0, 'Схема геодезических построений', 'Scheme_Geodesic_Plotting', 8, 4);
INSERT INTO class.section_mp VALUES (30, 0, 'Схема расположения земельных участков', 'Scheme_Disposition_Parcels', 9, 4);
INSERT INTO class.section_mp VALUES (31, 0, 'Чертеж земельных участков и их частей', 'Diagram_Parcels_SubParcels', 10, 4);
INSERT INTO class.section_mp VALUES (32, 0, 'Акт согласования местоположения границы ЗУ', 'Agreement_Document', 11, 4);
INSERT INTO class.section_mp VALUES (33, 0, 'Абрисы узловых точек границ ЗУ', 'NodalPointSchemes', 12, 4);
INSERT INTO class.section_mp VALUES (34, 0, 'Приложения', 'Appendix', 13, 4);


-- Update DataBase version information -----------------------------------------
UPDATE class.database_version SET (number_version, date_version) = (11, '2013-09-07');
